@@ -1,6 +1,6 @@
 // VIM_TEST_SETUP let g:java_highlight_functions = 'indent8'
+// VIM_TEST_SETUP let g:java_highlight_generics = 1
 // VIM_TEST_SETUP set encoding=utf-8 termencoding=utf-8
-
 import java.lang.annotation.ElementType;
 import java.lang.annotation.Target;
 
@@ -67,8 +67,8 @@ abstract class Indent8MethodsTests
         protected static final synchronized strictfp void ascii$99_()
         { ascii$98_(); }
 
-        @SuppressWarnings("strictfp")
-        protected static final synchronized strictfp <α, β> Τʬ<α>[] μʭʭ$99_(
+        @SuppressWarnings("strictfp") protected
+        static final synchronized strictfp <α, β> Τʬ<α>[] μʭʭ$99_(
                         java.util.function.Function<β, Τʬ<α>[][]> ƒ)
         {
                 return
@@ -77,6 +77,14 @@ abstract class Indent8MethodsTests
 
         public static Class<?> classLock() { return Indent8MethodsTests.class; }
 
+        public <E extends Comparable<? super E>> java.util.List<E> filter(
+                        java.util.function.Predicate<? super E> filter,
+                        java.util.List<? extends E> cs)
+        {
+                return cs.stream().filter(filter)
+                        .collect(java.util.stream.Collectors.toUnmodifiableList());
+        }
+
         @Override @SuppressWarnings("cast")
         public String toString() { return (String) "Indent8MethodsTests"; }
 }
