@@ -231,34 +231,32 @@ clip_update_selection(Clipboard_T *clip)
 }
 
     static int
-clip_gen_own_selection(Clipboard_T *cbd UNUSED)
+clip_gen_own_selection(Clipboard_T *cbd)
 {
-    if (clipmethod == CLIPMETHOD_GUI)
-    {
-#ifdef FEAT_GUI
-	if (gui.in_use)
-	    return clip_mch_own_selection(cbd);
-#endif
-    }
-    else if (clipmethod == CLIPMETHOD_WAYLAND)
-    {
-#ifdef FEAT_WAYLAND_CLIPBOARD
-	return clip_wl_own_selection(cbd);
-#endif
-    }
-    else if (clipmethod == CLIPMETHOD_X11)
-    {
-#ifdef FEAT_XCLIPBOARD
-	return clip_xterm_own_selection(cbd);
-#endif
-    }
-    else if (clipmethod == CLIPMETHOD_OTHER)
-    {
-#if !defined(FEAT_XCLIPBOARD) && !defined(FEAT_WAYLAND_CLIPBOARD)
+#if defined(FEAT_XCLIPBOARD) || defined(FEAT_WAYLAND_CLIPBOARD)
+# ifdef FEAT_GUI
+    if (gui.in_use)
 	return clip_mch_own_selection(cbd);
+    else
+# endif
+    {
+	if (clipmethod == CLIPMETHOD_WAYLAND)
+	{
+#ifdef FEAT_WAYLAND_CLIPBOARD
+	    return clip_wl_own_selection(cbd);
 #endif
+	}
+	else if (clipmethod == CLIPMETHOD_X11)
+	{
+#ifdef FEAT_XCLIPBOARD
+	    return clip_xterm_own_selection(cbd);
+#endif
+	}
     }
     return FAIL;
+#else
+    return clip_mch_own_selection(cbd);
+#endif
 }
 
     void
@@ -298,33 +296,31 @@ clip_own_selection(Clipboard_T *cbd)
 }
 
     static void
-clip_gen_lose_selection(Clipboard_T *cbd UNUSED)
+clip_gen_lose_selection(Clipboard_T *cbd)
 {
-    if (clipmethod == CLIPMETHOD_GUI)
-    {
-#ifdef FEAT_GUI
-	if (gui.in_use)
-	    clip_mch_lose_selection(cbd);
-#endif
-    }
-    else if (clipmethod == CLIPMETHOD_WAYLAND)
-    {
-#ifdef FEAT_WAYLAND_CLIPBOARD
-	clip_wl_lose_selection(cbd);
-#endif
-    }
-    else if (clipmethod == CLIPMETHOD_X11)
-    {
-#ifdef FEAT_XCLIPBOARD
-	clip_xterm_lose_selection(cbd);
-#endif
-    }
-    else if (clipmethod == CLIPMETHOD_OTHER)
-    {
-#if !defined(FEAT_XCLIPBOARD) && !defined(FEAT_WAYLAND_CLIPBOARD)
+#if defined(FEAT_XCLIPBOARD) || defined(FEAT_WAYLAND_CLIPBOARD)
+# ifdef FEAT_GUI
+    if (gui.in_use)
 	clip_mch_lose_selection(cbd);
+    else
+# endif
+    {
+	if (clipmethod == CLIPMETHOD_WAYLAND)
+	{
+#ifdef FEAT_WAYLAND_CLIPBOARD
+	    clip_wl_lose_selection(cbd);
 #endif
+	}
+	else if (clipmethod == CLIPMETHOD_X11)
+	{
+#ifdef FEAT_XCLIPBOARD
+	    clip_xterm_lose_selection(cbd);
+#endif
+	}
     }
+#else
+    clip_mch_lose_selection(cbd);
+#endif
 }
 
     void
@@ -1350,61 +1346,57 @@ clip_gen_set_selection(Clipboard_T *cbd)
 	    return;
 	}
     }
-    if (clipmethod == CLIPMETHOD_GUI)
-    {
-#ifdef FEAT_GUI
-	if (gui.in_use)
+#if defined(FEAT_XCLIPBOARD) || defined(FEAT_WAYLAND_CLIPBOARD)
+# ifdef FEAT_GUI
+    if (gui.in_use)
 	clip_mch_set_selection(cbd);
-#endif
-    }
-    else if (clipmethod == CLIPMETHOD_WAYLAND)
+    else
+# endif
     {
+	if (clipmethod == CLIPMETHOD_WAYLAND)
+	{
 #ifdef FEAT_WAYLAND_CLIPBOARD
-	clip_wl_set_selection(cbd);
+	    clip_wl_set_selection(cbd);
 #endif
-    }
-    else if (clipmethod == CLIPMETHOD_X11)
-    {
+	}
+	else if (clipmethod == CLIPMETHOD_X11)
+	{
 #ifdef FEAT_XCLIPBOARD
-	clip_xterm_set_selection(cbd);
+	    clip_xterm_set_selection(cbd);
 #endif
+	}
     }
-    else if (clipmethod == CLIPMETHOD_OTHER)
-    {
-#if !defined(FEAT_XCLIPBOARD) && !defined(FEAT_WAYLAND_CLIPBOARD)
-	clip_mch_set_selection(cbd);
+#else
+    clip_mch_set_selection(cbd);
 #endif
-    }
 }
 
     static void
-clip_gen_request_selection(Clipboard_T *cbd UNUSED)
+clip_gen_request_selection(Clipboard_T *cbd)
 {
-    if (clipmethod == CLIPMETHOD_GUI)
-    {
+#if defined(FEAT_XCLIPBOARD) || defined(FEAT_WAYLAND_CLIPBOARD)
 # ifdef FEAT_GUI
-	if (gui.in_use)
-	    clip_mch_request_selection(cbd);
-# endif
-    }
-    else if (clipmethod == CLIPMETHOD_WAYLAND)
-    {
-#ifdef FEAT_WAYLAND_CLIPBOARD
-	clip_wl_request_selection(cbd);
-#endif
-    }
-    else if (clipmethod == CLIPMETHOD_X11)
-    {
-#ifdef FEAT_XCLIPBOARD
-	clip_xterm_request_selection(cbd);
-#endif
-    }
-    else if (clipmethod == CLIPMETHOD_OTHER)
-    {
-#if !defined(FEAT_XCLIPBOARD) && !defined(FEAT_WAYLAND_CLIPBOARD)
+    if (gui.in_use)
 	clip_mch_request_selection(cbd);
+    else
+# endif
+    {
+	if (clipmethod == CLIPMETHOD_WAYLAND)
+	{
+#ifdef FEAT_WAYLAND_CLIPBOARD
+	    clip_wl_request_selection(cbd);
 #endif
+	}
+	else if (clipmethod == CLIPMETHOD_X11)
+	{
+#ifdef FEAT_XCLIPBOARD
+	    clip_xterm_request_selection(cbd);
+#endif
+	}
     }
+#else
+    clip_mch_request_selection(cbd);
+#endif
 }
 
 #if (defined(FEAT_X11) && defined(FEAT_XCLIPBOARD) && defined(USE_SYSTEM)) \
@@ -1421,28 +1413,31 @@ clip_x11_owner_exists(Clipboard_T *cbd)
     int
 clip_gen_owner_exists(Clipboard_T *cbd UNUSED)
 {
-    if (clipmethod == CLIPMETHOD_OTHER)
-    {
+#if defined(FEAT_XCLIPBOARD) || defined(FEAT_WAYLAND_CLIPBOARD)
 # ifdef FEAT_GUI_GTK
-	if (gui.in_use)
-	    return clip_gtk_owner_exists(cbd);
-# endif
-    }
-    else if (clipmethod == CLIPMETHOD_WAYLAND)
-    {
-# ifdef FEAT_WAYLAND_CLIPBOARD
-	return clip_wl_owner_exists(cbd);
-# endif
-    }
-    else if (clipmethod == CLIPMETHOD_X11)
-    {
-# ifdef FEAT_XCLIPBOARD
-	return clip_x11_owner_exists(cbd);
-# endif
-    }
+    if (gui.in_use)
+	return clip_gtk_owner_exists(cbd);
     else
-	return FALSE;
-    return FALSE;
+# endif
+    {
+	if (clipmethod == CLIPMETHOD_WAYLAND)
+	{
+#ifdef FEAT_WAYLAND_CLIPBOARD
+	    return clip_wl_owner_exists(cbd);
+#endif
+	}
+	else if (clipmethod == CLIPMETHOD_X11)
+	{
+#ifdef FEAT_XCLIPBOARD
+	    return clip_x11_owner_exists(cbd);
+#endif
+	}
+	else
+	    return FALSE;
+    }
+#else
+    return TRUE;
+#endif
 }
 #endif
 
@@ -3440,7 +3435,7 @@ clip_wl_owner_exists(Clipboard_T *cbd)
  * depending on the order of values in str.
  */
     static clipmethod_T
-get_clipmethod(char_u *str, bool *regular, bool *primary)
+get_clipmethod(char_u *str, bool *plus UNUSED, bool *star UNUSED)
 {
     int		len	= (int)STRLEN(str) + 1;
     char_u	*buf	= alloc(len);
@@ -3467,8 +3462,8 @@ get_clipmethod(char_u *str, bool *regular, bool *primary)
 		if (clip_wl.regular.available || clip_wl.primary.available)
 		{
 		    method = CLIPMETHOD_WAYLAND;
-		    *regular = clip_wl.regular.available;
-		    *primary = clip_wl.primary.available;
+		    *plus = clip_wl.regular.available;
+		    *star = clip_wl.primary.available;
 		}
 #endif
 	    }
@@ -3480,38 +3475,21 @@ get_clipmethod(char_u *str, bool *regular, bool *primary)
 #endif
 	    {
 #ifdef FEAT_XCLIPBOARD
-		// x_IOerror_handler() in os_unix.c should set xterm_dpy to NULL
-		// if we lost connection to the X server.
+		// x_IOerror_handler() in os_unix.c should set xterm_dpy to NULL if
+		// we lost connection to the X server.
 		if (xterm_dpy != NULL)
 		{
-		    // If the X connection is lost then that handler will
-		    // longjmp somewhere else, in that case we will call
-		    // choose_clipmethod() again from there, and this if block
-		    // won't be executed since xterm_dpy will be set to NULL.
+		    // If the X connection is lost then that handler will longjmp
+		    // somewhere else, in that case we will call choose_clipmethod()
+		    // again from there, and this if block won't be executed since
+		    // xterm_dpy will be set to NULL.
 		    xterm_update();
 		    method = CLIPMETHOD_X11;
-		    *regular = *primary = true;
+		    *plus = *star = TRUE;
 		}
 #endif
 	    }
 	}
-	else if (STRCMP(buf, "gui") == 0)
-	{
-#ifdef FEAT_GUI
-	    if (gui.in_use)
-	    {
-		method = CLIPMETHOD_GUI;
-		*regular = *primary = true;
-	    }
-#endif
-	}
-	else if (STRCMP(buf, "other") == 0)
-	{
-#if !defined(FEAT_XCLIPBOARD) && !defined(FEAT_WAYLAND_CLIPBOARD)
-		method = CLIPMETHOD_OTHER;
-		*regular = *primary = true;
-#endif
-	}
 	else
 	{
 	    ret = CLIPMETHOD_FAIL;
@@ -3535,21 +3513,17 @@ exit:
 /*
  * Returns name of clipmethod in a statically allocated string.
  */
-    static char_u *
+    static char *
 clipmethod_to_str(clipmethod_T method)
 {
     switch(method)
     {
 	case CLIPMETHOD_WAYLAND:
-	    return (char_u *)"wayland";
+	    return "wayland";
 	case CLIPMETHOD_X11:
-	    return (char_u *)"x11";
-	case CLIPMETHOD_GUI:
-	    return (char_u *)"gui";
-	case CLIPMETHOD_OTHER:
-	    return (char_u *)"other";
+	    return "x11";
 	default:
-	    return (char_u *)"none";
+	    return "none";
     }
 }
 
@@ -3566,11 +3540,27 @@ choose_clipmethod(void)
     if (method == CLIPMETHOD_FAIL)
 	return e_invalid_argument;
 
-#if defined(FEAT_GUI) && defined(FEAT_WAYLAND)
-    if (method == CLIPMETHOD_GUI)
+// If GUI is running or we are not on a system with Wayland or X11, then always
+// return CLIPMETHOD_NONE. System or GUI clipboard handling always overrides.
+#if defined(FEAT_XCLIPBOARD) || defined(FEAT_WAYLAND_CLIPBOARD)
+# if defined(FEAT_GUI)
+    if (gui.in_use)
+    {
+#  ifdef FEAT_WAYLAND
 	// We only interact with Wayland for the clipboard, we can just deinit
 	// everything.
 	wayland_uninit_connection();
+#  endif
+
+	method = CLIPMETHOD_NONE;
+	goto lose_sel_exit;
+    }
+# endif
+#else
+    // If on a system like windows or macos, then clipmethod is irrelevant, we
+    // use their way of accessing the clipboard.
+    method = CLIPMETHOD_NONE;
+    goto exit;
 #endif
 
     // Deinitialize clipboard if there is no way to access clipboard
@@ -3587,18 +3577,32 @@ choose_clipmethod(void)
     // Disown clipboard if we are switching to a new method
     else if (clipmethod != CLIPMETHOD_NONE && method != clipmethod)
     {
+#if (defined(FEAT_XCLIPBOARD) || defined(FEAT_WAYLAND_CLIPBOARD)) \
+	&& defined(FEAT_GUI)
+lose_sel_exit:
+#endif
 	if (clip_star.owned)
 	    clip_lose_selection(&clip_star);
 	if (clip_plus.owned)
 	    clip_lose_selection(&clip_plus);
-	clip_init_single(&clip_plus, regular);
-	clip_init_single(&clip_star, primary);
+
+#if defined(FEAT_GUI)
+	if (!gui.in_use)
+#endif
+	{
+	    clip_init_single(&clip_plus, regular);
+	    clip_init_single(&clip_star, primary);
+	}
     }
 
+#if !defined(FEAT_XCLIPBOARD) && !defined(FEAT_WAYLAND_CLIPBOARD)
+exit:
+#endif
+
     clipmethod = method;
 
 #ifdef FEAT_EVAL
-    set_vim_var_string(VV_CLIPMETHOD, clipmethod_to_str(method), -1);
+    set_vim_var_string(VV_CLIPMETHOD, (char_u*)clipmethod_to_str(method), -1);
 #endif
 
     return NULL;
