@@ -150,9 +150,7 @@
  */
 enum key_extra
 {
-    KE_NAME = 3		// name of this terminal entry
-
-    , KE_S_UP = 4		// shift-up
+      KE_S_UP = 4		// shift-up
     , KE_S_DOWN = 5		// shift-down
 
     , KE_S_F1 = 6		// shifted function keys
@@ -253,7 +251,8 @@ enum key_extra
     , KE_CSI = 81		// CSI typed directly
     , KE_SNR = 82		// <SNR>
     , KE_PLUG = 83		// <Plug>
-    , KE_CMDWIN = 84		// open command-line window from Command-line Mode
+    , KE_CMDWIN = 84		// open command-line window from Command-line
+				// Mode
 
     , KE_C_LEFT = 85		// control-left
     , KE_C_RIGHT = 86		// control-right
@@ -276,13 +275,20 @@ enum key_extra
     , KE_MOUSEMOVE_XY = 101	// KE_MOUSEMOVE with coordinates
     , KE_CANCEL = 102		// return from vgetc()
     , KE_COMMAND = 103		// <Cmd> special key
+    , KE_SCRIPT_COMMAND = 104	// <ScriptCmd> special key
+    , KE_S_BS = 105		// shift + <BS>
+    , KE_SID = 106		// <SID> special key, followed by {nr};
+    , KE_ESC = 107		// used for K_ESC
+    , KE_WILD = 108		// triggers wildmode completion
 };
 
 /*
- * the three byte codes are replaced with the following int when using vgetc()
+ * The three-byte codes are replaced with a negative number when using vgetc().
  */
 #define K_ZERO		TERMCAP2KEY(KS_ZERO, KE_FILLER)
 
+#define K_ESC		TERMCAP2KEY(KS_EXTRA, KE_ESC)
+
 #define K_UP		TERMCAP2KEY('k', 'u')
 #define K_DOWN		TERMCAP2KEY('k', 'd')
 #define K_LEFT		TERMCAP2KEY('k', 'l')
@@ -293,12 +299,15 @@ enum key_extra
 #define K_C_LEFT	TERMCAP2KEY(KS_EXTRA, KE_C_LEFT)
 #define K_S_RIGHT	TERMCAP2KEY('%', 'i')
 #define K_C_RIGHT	TERMCAP2KEY(KS_EXTRA, KE_C_RIGHT)
+
 #define K_S_HOME	TERMCAP2KEY('#', '2')
 #define K_C_HOME	TERMCAP2KEY(KS_EXTRA, KE_C_HOME)
 #define K_S_END		TERMCAP2KEY('*', '7')
 #define K_C_END		TERMCAP2KEY(KS_EXTRA, KE_C_END)
+
 #define K_TAB		TERMCAP2KEY(KS_EXTRA, KE_TAB)
 #define K_S_TAB		TERMCAP2KEY('k', 'B')
+#define K_S_BS		TERMCAP2KEY(KS_EXTRA, KE_S_BS)
 
 // extra set of function keys F1-F4, for vt100 compatible xterm
 #define K_XF1		TERMCAP2KEY(KS_EXTRA, KE_XF1)
@@ -480,6 +489,10 @@ enum key_extra
 #define K_CURSORHOLD	TERMCAP2KEY(KS_EXTRA, KE_CURSORHOLD)
 
 #define K_COMMAND	TERMCAP2KEY(KS_EXTRA, KE_COMMAND)
+#define K_SCRIPT_COMMAND TERMCAP2KEY(KS_EXTRA, KE_SCRIPT_COMMAND)
+#define K_SID		TERMCAP2KEY(KS_EXTRA, KE_SID)
+
+#define K_WILD		TERMCAP2KEY(KS_EXTRA, KE_WILD)
 
 // Bits for modifier mask
 // 0x01 cannot be used, because the modifier must be 0x02 or higher
@@ -490,8 +503,8 @@ enum key_extra
 #define MOD_MASK_2CLICK	    0x20	// use MOD_MASK_MULTI_CLICK
 #define MOD_MASK_3CLICK	    0x40	// use MOD_MASK_MULTI_CLICK
 #define MOD_MASK_4CLICK	    0x60	// use MOD_MASK_MULTI_CLICK
-#ifdef MACOS_X
-# define MOD_MASK_CMD	    0x80
+#if defined(MACOS_X) || defined(FEAT_GUI_GTK)
+# define MOD_MASK_CMD	    0x80        // aka SUPER
 #endif
 
 #define MOD_MASK_MULTI_CLICK	(MOD_MASK_2CLICK|MOD_MASK_3CLICK|MOD_MASK_4CLICK)
