@@ -60,11 +60,8 @@ coladvance_force(colnr_T wcol)
     if (wcol == MAXCOL)
 	curwin->w_valid &= ~VALID_VIRTCOL;
     else
-    {
 	// Virtcol is valid
-	curwin->w_valid |= VALID_VIRTCOL;
-	curwin->w_virtcol = wcol;
-    }
+	set_valid_virtcol(curwin, wcol);
     return rc;
 }
 
@@ -101,11 +98,8 @@ coladvance(colnr_T wantcol)
     if (wantcol == MAXCOL || rc == FAIL)
 	curwin->w_valid &= ~VALID_VIRTCOL;
     else if (*ml_get_cursor() != TAB)
-    {
 	// Virtcol is valid when not on a TAB
-	curwin->w_valid |= VALID_VIRTCOL;
-	curwin->w_virtcol = wantcol;
-    }
+	set_valid_virtcol(curwin, wantcol);
     return rc;
 }
 
@@ -771,7 +765,7 @@ copy_option_part(
     return len;
 }
 
-#ifndef HAVE_MEMSET
+#if !defined(HAVE_MEMSET) && !defined(PROTO)
     void *
 vim_memset(void *ptr, int c, size_t size)
 {
@@ -921,16 +915,6 @@ static char_u modifier_keys_table[] =
     NUL
 };
 
-// Indexes into the key_names_table array
-// Must be kept in sync with the array!
-#define IDX_KEYNAME_BS 2
-#define IDX_KEYNAME_CAR 5
-#define IDX_KEYNAME_DEL 9
-#define IDX_KEYNAME_INS 59
-#define IDX_KEYNAME_NL 101
-#define IDX_KEYNAME_SWD 115
-#define IDX_KEYNAME_SWU 118
-
 #define STRING_INIT(s) \
     {(char_u *)(s), STRLEN_LITERAL(s)}
 static struct key_name_entry
@@ -938,210 +922,199 @@ static struct key_name_entry
     int		enabled;	    // is this entry available (TRUE/FALSE)?
     int		key;		    // special key code or ascii value
     string_T	name;		    // name of key
-    string_T	*pref_name;	    // pointer to preferred key name
-				    // (may be NULL or point to the name in another entry)
+    int		is_alt;		    // is an alternative name
 } key_names_table[] =
 // Must be sorted by the 'name.string' field in ascending order because it is used by bsearch()!
 {
-    {TRUE, K_BS, STRING_INIT("BackSpace"),
-	&key_names_table[IDX_KEYNAME_BS].name},			// Alternative name
-    {TRUE, '|', STRING_INIT("Bar"), NULL},
-    {TRUE, K_BS, STRING_INIT("BS"), NULL},
-    {TRUE, '\\', STRING_INIT("Bslash"), NULL},
-    {TRUE, K_COMMAND, STRING_INIT("Cmd"), NULL},
-    {TRUE, CAR, STRING_INIT("CR"), NULL},
-    {TRUE, CSI, STRING_INIT("CSI"), NULL},
-    {TRUE, K_CURSORHOLD, STRING_INIT("CursorHold"), NULL},
+    {TRUE, K_BS, STRING_INIT("BackSpace"), TRUE},
+    {TRUE, '|', STRING_INIT("Bar"), FALSE},
+    {TRUE, K_BS, STRING_INIT("BS"), FALSE},
+    {TRUE, '\\', STRING_INIT("Bslash"), FALSE},
+    {TRUE, K_COMMAND, STRING_INIT("Cmd"), FALSE},
+    {TRUE, CAR, STRING_INIT("CR"), FALSE},
+    {TRUE, CSI, STRING_INIT("CSI"), FALSE},
+    {TRUE, K_CURSORHOLD, STRING_INIT("CursorHold"), FALSE},
     {
 #ifdef FEAT_MOUSE_DEC
     TRUE,
 #else
     FALSE,
 #endif
-	K_DEC_MOUSE, STRING_INIT("DecMouse"), NULL},
-    {TRUE, K_DEL, STRING_INIT("Del"), NULL},
-    {TRUE, K_DEL, STRING_INIT("Delete"),
-	&key_names_table[IDX_KEYNAME_DEL].name},		// Alternative name
-    {TRUE, K_DOWN, STRING_INIT("Down"), NULL},
-    {TRUE, K_DROP, STRING_INIT("Drop"), NULL},
-    {TRUE, K_END, STRING_INIT("End"), NULL},
-    {TRUE, CAR, STRING_INIT("Enter"),
-	&key_names_table[IDX_KEYNAME_CAR].name},		// Alternative name
-    {TRUE, ESC, STRING_INIT("Esc"), NULL},
+	K_DEC_MOUSE, STRING_INIT("DecMouse"), FALSE},
+    {TRUE, K_DEL, STRING_INIT("Del"), FALSE},
+    {TRUE, K_DEL, STRING_INIT("Delete"), TRUE},
+    {TRUE, K_DOWN, STRING_INIT("Down"), FALSE},
+    {TRUE, K_DROP, STRING_INIT("Drop"), FALSE},
+    {TRUE, K_END, STRING_INIT("End"), FALSE},
+    {TRUE, CAR, STRING_INIT("Enter"), TRUE},
+    {TRUE, ESC, STRING_INIT("Esc"), FALSE},
 
-    {TRUE, K_F1, STRING_INIT("F1"), NULL},
-    {TRUE, K_F10, STRING_INIT("F10"), NULL},
-    {TRUE, K_F11, STRING_INIT("F11"), NULL},
-    {TRUE, K_F12, STRING_INIT("F12"), NULL},
-    {TRUE, K_F13, STRING_INIT("F13"), NULL},
-    {TRUE, K_F14, STRING_INIT("F14"), NULL},
-    {TRUE, K_F15, STRING_INIT("F15"), NULL},
-    {TRUE, K_F16, STRING_INIT("F16"), NULL},
-    {TRUE, K_F17, STRING_INIT("F17"), NULL},
-    {TRUE, K_F18, STRING_INIT("F18"), NULL},
-    {TRUE, K_F19, STRING_INIT("F19"), NULL},
+    {TRUE, K_F1, STRING_INIT("F1"), FALSE},
+    {TRUE, K_F10, STRING_INIT("F10"), FALSE},
+    {TRUE, K_F11, STRING_INIT("F11"), FALSE},
+    {TRUE, K_F12, STRING_INIT("F12"), FALSE},
+    {TRUE, K_F13, STRING_INIT("F13"), FALSE},
+    {TRUE, K_F14, STRING_INIT("F14"), FALSE},
+    {TRUE, K_F15, STRING_INIT("F15"), FALSE},
+    {TRUE, K_F16, STRING_INIT("F16"), FALSE},
+    {TRUE, K_F17, STRING_INIT("F17"), FALSE},
+    {TRUE, K_F18, STRING_INIT("F18"), FALSE},
+    {TRUE, K_F19, STRING_INIT("F19"), FALSE},
 
-    {TRUE, K_F2, STRING_INIT("F2"), NULL},
-    {TRUE, K_F20, STRING_INIT("F20"), NULL},
-    {TRUE, K_F21, STRING_INIT("F21"), NULL},
-    {TRUE, K_F22, STRING_INIT("F22"), NULL},
-    {TRUE, K_F23, STRING_INIT("F23"), NULL},
-    {TRUE, K_F24, STRING_INIT("F24"), NULL},
-    {TRUE, K_F25, STRING_INIT("F25"), NULL},
-    {TRUE, K_F26, STRING_INIT("F26"), NULL},
-    {TRUE, K_F27, STRING_INIT("F27"), NULL},
-    {TRUE, K_F28, STRING_INIT("F28"), NULL},
-    {TRUE, K_F29, STRING_INIT("F29"), NULL},
+    {TRUE, K_F2, STRING_INIT("F2"), FALSE},
+    {TRUE, K_F20, STRING_INIT("F20"), FALSE},
+    {TRUE, K_F21, STRING_INIT("F21"), FALSE},
+    {TRUE, K_F22, STRING_INIT("F22"), FALSE},
+    {TRUE, K_F23, STRING_INIT("F23"), FALSE},
+    {TRUE, K_F24, STRING_INIT("F24"), FALSE},
+    {TRUE, K_F25, STRING_INIT("F25"), FALSE},
+    {TRUE, K_F26, STRING_INIT("F26"), FALSE},
+    {TRUE, K_F27, STRING_INIT("F27"), FALSE},
+    {TRUE, K_F28, STRING_INIT("F28"), FALSE},
+    {TRUE, K_F29, STRING_INIT("F29"), FALSE},
 
-    {TRUE, K_F3, STRING_INIT("F3"), NULL},
-    {TRUE, K_F30, STRING_INIT("F30"), NULL},
-    {TRUE, K_F31, STRING_INIT("F31"), NULL},
-    {TRUE, K_F32, STRING_INIT("F32"), NULL},
-    {TRUE, K_F33, STRING_INIT("F33"), NULL},
-    {TRUE, K_F34, STRING_INIT("F34"), NULL},
-    {TRUE, K_F35, STRING_INIT("F35"), NULL},
-    {TRUE, K_F36, STRING_INIT("F36"), NULL},
-    {TRUE, K_F37, STRING_INIT("F37"), NULL},
+    {TRUE, K_F3, STRING_INIT("F3"), FALSE},
+    {TRUE, K_F30, STRING_INIT("F30"), FALSE},
+    {TRUE, K_F31, STRING_INIT("F31"), FALSE},
+    {TRUE, K_F32, STRING_INIT("F32"), FALSE},
+    {TRUE, K_F33, STRING_INIT("F33"), FALSE},
+    {TRUE, K_F34, STRING_INIT("F34"), FALSE},
+    {TRUE, K_F35, STRING_INIT("F35"), FALSE},
+    {TRUE, K_F36, STRING_INIT("F36"), FALSE},
+    {TRUE, K_F37, STRING_INIT("F37"), FALSE},
 
-    {TRUE, K_F4, STRING_INIT("F4"), NULL},
-    {TRUE, K_F5, STRING_INIT("F5"), NULL},
-    {TRUE, K_F6, STRING_INIT("F6"), NULL},
-    {TRUE, K_F7, STRING_INIT("F7"), NULL},
-    {TRUE, K_F8, STRING_INIT("F8"), NULL},
-    {TRUE, K_F9, STRING_INIT("F9"), NULL},
+    {TRUE, K_F4, STRING_INIT("F4"), FALSE},
+    {TRUE, K_F5, STRING_INIT("F5"), FALSE},
+    {TRUE, K_F6, STRING_INIT("F6"), FALSE},
+    {TRUE, K_F7, STRING_INIT("F7"), FALSE},
+    {TRUE, K_F8, STRING_INIT("F8"), FALSE},
+    {TRUE, K_F9, STRING_INIT("F9"), FALSE},
 
-    {TRUE, K_FOCUSGAINED, STRING_INIT("FocusGained"), NULL},
-    {TRUE, K_FOCUSLOST, STRING_INIT("FocusLost"), NULL},
-    {TRUE, K_HELP, STRING_INIT("Help"), NULL},
-    {TRUE, K_HOME, STRING_INIT("Home"), NULL},
-    {TRUE, K_IGNORE, STRING_INIT("Ignore"), NULL},
-    {TRUE, K_INS, STRING_INIT("Ins"),
-	&key_names_table[IDX_KEYNAME_INS].name},		// Alternative name
-    {TRUE, K_INS, STRING_INIT("Insert"), NULL},
+    {TRUE, K_FOCUSGAINED, STRING_INIT("FocusGained"), FALSE},
+    {TRUE, K_FOCUSLOST, STRING_INIT("FocusLost"), FALSE},
+    {TRUE, K_HELP, STRING_INIT("Help"), FALSE},
+    {TRUE, K_HOME, STRING_INIT("Home"), FALSE},
+    {TRUE, K_IGNORE, STRING_INIT("Ignore"), FALSE},
+    {TRUE, K_INS, STRING_INIT("Ins"), TRUE},
+    {TRUE, K_INS, STRING_INIT("Insert"), FALSE},
     {
 #ifdef FEAT_MOUSE_JSB
     TRUE,
 #else
     FALSE,
 #endif
-	K_JSBTERM_MOUSE, STRING_INIT("JsbMouse"), NULL},
-    {TRUE, K_K0, STRING_INIT("k0"), NULL},
-    {TRUE, K_K1, STRING_INIT("k1"), NULL},
-    {TRUE, K_K2, STRING_INIT("k2"), NULL},
-    {TRUE, K_K3, STRING_INIT("k3"), NULL},
-    {TRUE, K_K4, STRING_INIT("k4"), NULL},
-    {TRUE, K_K5, STRING_INIT("k5"), NULL},
-    {TRUE, K_K6, STRING_INIT("k6"), NULL},
-    {TRUE, K_K7, STRING_INIT("k7"), NULL},
-    {TRUE, K_K8, STRING_INIT("k8"), NULL},
-    {TRUE, K_K9, STRING_INIT("k9"), NULL},
+	K_JSBTERM_MOUSE, STRING_INIT("JsbMouse"), FALSE},
+    {TRUE, K_K0, STRING_INIT("k0"), FALSE},
+    {TRUE, K_K1, STRING_INIT("k1"), FALSE},
+    {TRUE, K_K2, STRING_INIT("k2"), FALSE},
+    {TRUE, K_K3, STRING_INIT("k3"), FALSE},
+    {TRUE, K_K4, STRING_INIT("k4"), FALSE},
+    {TRUE, K_K5, STRING_INIT("k5"), FALSE},
+    {TRUE, K_K6, STRING_INIT("k6"), FALSE},
+    {TRUE, K_K7, STRING_INIT("k7"), FALSE},
+    {TRUE, K_K8, STRING_INIT("k8"), FALSE},
+    {TRUE, K_K9, STRING_INIT("k9"), FALSE},
 
-    {TRUE, K_KDEL, STRING_INIT("kDel"), NULL},
-    {TRUE, K_KDIVIDE, STRING_INIT("kDivide"), NULL},
-    {TRUE, K_KEND, STRING_INIT("kEnd"), NULL},
-    {TRUE, K_KENTER, STRING_INIT("kEnter"), NULL},
-    {TRUE, K_KHOME, STRING_INIT("kHome"), NULL},
-    {TRUE, K_KINS, STRING_INIT("kInsert"), NULL},
-    {TRUE, K_KMINUS, STRING_INIT("kMinus"), NULL},
-    {TRUE, K_KMULTIPLY, STRING_INIT("kMultiply"), NULL},
-    {TRUE, K_KPAGEDOWN, STRING_INIT("kPageDown"), NULL},
-    {TRUE, K_KPAGEUP, STRING_INIT("kPageUp"), NULL},
-    {TRUE, K_KPLUS, STRING_INIT("kPlus"), NULL},
-    {TRUE, K_KPOINT, STRING_INIT("kPoint"), NULL},
-    {TRUE, K_LEFT, STRING_INIT("Left"), NULL},
-    {TRUE, K_LEFTDRAG, STRING_INIT("LeftDrag"), NULL},
-    {TRUE, K_LEFTMOUSE, STRING_INIT("LeftMouse"), NULL},
-    {TRUE, K_LEFTMOUSE_NM, STRING_INIT("LeftMouseNM"), NULL},
-    {TRUE, K_LEFTRELEASE, STRING_INIT("LeftRelease"), NULL},
-    {TRUE, K_LEFTRELEASE_NM, STRING_INIT("LeftReleaseNM"), NULL},
-    {TRUE, NL, STRING_INIT("LF"),
-	&key_names_table[IDX_KEYNAME_NL].name},			    // Alternative name
-    {TRUE, NL, STRING_INIT("LineFeed"),
-	&key_names_table[IDX_KEYNAME_NL].name},			    // Alternative name
-    {TRUE, '<', STRING_INIT("lt"), NULL},
-    {TRUE, K_MIDDLEDRAG, STRING_INIT("MiddleDrag"), NULL},
-    {TRUE, K_MIDDLEMOUSE, STRING_INIT("MiddleMouse"), NULL},
-    {TRUE, K_MIDDLERELEASE, STRING_INIT("MiddleRelease"), NULL},
-    {TRUE, K_MOUSE, STRING_INIT("Mouse"), NULL},
-    {TRUE, K_MOUSEDOWN, STRING_INIT("MouseDown"),
-	&key_names_table[IDX_KEYNAME_SWU].name},		    // OBSOLETE: Use ScrollWheelUp instead
-    {TRUE, K_MOUSEMOVE, STRING_INIT("MouseMove"), NULL},
-    {TRUE, K_MOUSEUP, STRING_INIT("MouseUp"),
-	&key_names_table[IDX_KEYNAME_SWD].name},		    // OBSELETE: Use ScrollWheelDown instead
+    {TRUE, K_KDEL, STRING_INIT("kDel"), FALSE},
+    {TRUE, K_KDIVIDE, STRING_INIT("kDivide"), FALSE},
+    {TRUE, K_KEND, STRING_INIT("kEnd"), FALSE},
+    {TRUE, K_KENTER, STRING_INIT("kEnter"), FALSE},
+    {TRUE, K_KHOME, STRING_INIT("kHome"), FALSE},
+    {TRUE, K_KINS, STRING_INIT("kInsert"), FALSE},
+    {TRUE, K_KMINUS, STRING_INIT("kMinus"), FALSE},
+    {TRUE, K_KMULTIPLY, STRING_INIT("kMultiply"), FALSE},
+    {TRUE, K_KPAGEDOWN, STRING_INIT("kPageDown"), FALSE},
+    {TRUE, K_KPAGEUP, STRING_INIT("kPageUp"), FALSE},
+    {TRUE, K_KPLUS, STRING_INIT("kPlus"), FALSE},
+    {TRUE, K_KPOINT, STRING_INIT("kPoint"), FALSE},
+    {TRUE, K_LEFT, STRING_INIT("Left"), FALSE},
+    {TRUE, K_LEFTDRAG, STRING_INIT("LeftDrag"), FALSE},
+    {TRUE, K_LEFTMOUSE, STRING_INIT("LeftMouse"), FALSE},
+    {TRUE, K_LEFTMOUSE_NM, STRING_INIT("LeftMouseNM"), FALSE},
+    {TRUE, K_LEFTRELEASE, STRING_INIT("LeftRelease"), FALSE},
+    {TRUE, K_LEFTRELEASE_NM, STRING_INIT("LeftReleaseNM"), FALSE},
+    {TRUE, NL, STRING_INIT("LF"), TRUE},
+    {TRUE, NL, STRING_INIT("LineFeed"), TRUE},
+    {TRUE, '<', STRING_INIT("lt"), FALSE},
+    {TRUE, K_MIDDLEDRAG, STRING_INIT("MiddleDrag"), FALSE},
+    {TRUE, K_MIDDLEMOUSE, STRING_INIT("MiddleMouse"), FALSE},
+    {TRUE, K_MIDDLERELEASE, STRING_INIT("MiddleRelease"), FALSE},
+    {TRUE, K_MOUSE, STRING_INIT("Mouse"), FALSE},
+    {TRUE, K_MOUSEDOWN, STRING_INIT("MouseDown"), TRUE},
+    {TRUE, K_MOUSEMOVE, STRING_INIT("MouseMove"), FALSE},
+    {TRUE, K_MOUSEUP, STRING_INIT("MouseUp"), TRUE},
     {
 #ifdef FEAT_MOUSE_NET
     TRUE,
 #else
     FALSE,
 #endif
-	K_NETTERM_MOUSE, STRING_INIT("NetMouse"), NULL},
-    {TRUE, NL, STRING_INIT("NewLine"),
-	&key_names_table[IDX_KEYNAME_NL].name},			    // Alternative name
-    {TRUE, NL, STRING_INIT("NL"), NULL},
-    {TRUE, K_ZERO, STRING_INIT("Nul"), NULL},
-    {TRUE, K_PAGEDOWN, STRING_INIT("PageDown"), NULL},
-    {TRUE, K_PAGEUP, STRING_INIT("PageUp"), NULL},
-    {TRUE, K_PE, STRING_INIT("PasteEnd"), NULL},
-    {TRUE, K_PS, STRING_INIT("PasteStart"), NULL},
-    {TRUE, K_PLUG, STRING_INIT("Plug"), NULL},
+	K_NETTERM_MOUSE, STRING_INIT("NetMouse"), FALSE},
+    {TRUE, NL, STRING_INIT("NewLine"), TRUE},
+    {TRUE, NL, STRING_INIT("NL"), FALSE},
+    {TRUE, K_ZERO, STRING_INIT("Nul"), FALSE},
+    {TRUE, K_PAGEDOWN, STRING_INIT("PageDown"), FALSE},
+    {TRUE, K_PAGEUP, STRING_INIT("PageUp"), FALSE},
+    {TRUE, K_PE, STRING_INIT("PasteEnd"), FALSE},
+    {TRUE, K_PS, STRING_INIT("PasteStart"), FALSE},
+    {TRUE, K_PLUG, STRING_INIT("Plug"), FALSE},
     {
 #ifdef FEAT_MOUSE_PTERM
     TRUE,
 #else
     FALSE,
 #endif
-	K_PTERM_MOUSE, STRING_INIT("PtermMouse"), NULL},
-    {TRUE, CAR, STRING_INIT("Return"),
-	&key_names_table[IDX_KEYNAME_CAR].name},		    // Alternative name
-    {TRUE, K_RIGHT, STRING_INIT("Right"), NULL},
-    {TRUE, K_RIGHTDRAG, STRING_INIT("RightDrag"), NULL},
-    {TRUE, K_RIGHTMOUSE, STRING_INIT("RightMouse"), NULL},
-    {TRUE, K_RIGHTRELEASE, STRING_INIT("RightRelease"), NULL},
-    {TRUE, K_SCRIPT_COMMAND, STRING_INIT("ScriptCmd"), NULL},
-    {TRUE, K_MOUSEUP, STRING_INIT("ScrollWheelDown"), NULL},
-    {TRUE, K_MOUSERIGHT, STRING_INIT("ScrollWheelLeft"), NULL},
-    {TRUE, K_MOUSELEFT, STRING_INIT("ScrollWheelRight"), NULL},
-    {TRUE, K_MOUSEDOWN, STRING_INIT("ScrollWheelUp"), NULL},
-    {TRUE, K_SGR_MOUSE, STRING_INIT("SgrMouse"), NULL},
-    {TRUE, K_SGR_MOUSERELEASE, STRING_INIT("SgrMouseRelease"), NULL},
+	K_PTERM_MOUSE, STRING_INIT("PtermMouse"), FALSE},
+    {TRUE, CAR, STRING_INIT("Return"), TRUE},
+    {TRUE, K_RIGHT, STRING_INIT("Right"), FALSE},
+    {TRUE, K_RIGHTDRAG, STRING_INIT("RightDrag"), FALSE},
+    {TRUE, K_RIGHTMOUSE, STRING_INIT("RightMouse"), FALSE},
+    {TRUE, K_RIGHTRELEASE, STRING_INIT("RightRelease"), FALSE},
+    {TRUE, K_SCRIPT_COMMAND, STRING_INIT("ScriptCmd"), FALSE},
+    {TRUE, K_MOUSEUP, STRING_INIT("ScrollWheelDown"), FALSE},
+    {TRUE, K_MOUSERIGHT, STRING_INIT("ScrollWheelLeft"), FALSE},
+    {TRUE, K_MOUSELEFT, STRING_INIT("ScrollWheelRight"), FALSE},
+    {TRUE, K_MOUSEDOWN, STRING_INIT("ScrollWheelUp"), FALSE},
+    {TRUE, K_SGR_MOUSE, STRING_INIT("SgrMouse"), FALSE},
+    {TRUE, K_SGR_MOUSERELEASE, STRING_INIT("SgrMouseRelease"), FALSE},
     {
 #ifdef FEAT_EVAL
     TRUE,
 #else
     FALSE,
 #endif
-	K_SNR, STRING_INIT("SNR"), NULL},
-    {TRUE, ' ', STRING_INIT("Space"), NULL},
-    {TRUE, TAB, STRING_INIT("Tab"), NULL},
-    {TRUE, K_TAB, STRING_INIT("Tab"), NULL},
-    {TRUE, K_UNDO, STRING_INIT("Undo"), NULL},
-    {TRUE, K_UP, STRING_INIT("Up"), NULL},
+	K_SNR, STRING_INIT("SNR"), FALSE},
+    {TRUE, ' ', STRING_INIT("Space"), FALSE},
+    {TRUE, TAB, STRING_INIT("Tab"), FALSE},
+    {TRUE, K_TAB, STRING_INIT("Tab"), FALSE},
+    {TRUE, K_UNDO, STRING_INIT("Undo"), FALSE},
+    {TRUE, K_UP, STRING_INIT("Up"), FALSE},
     {
 #ifdef FEAT_MOUSE_URXVT
     TRUE,
 #else
     FALSE,
 #endif
-	K_URXVT_MOUSE, STRING_INIT("UrxvtMouse"), NULL},
-    {TRUE, K_X1DRAG, STRING_INIT("X1Drag"), NULL},
-    {TRUE, K_X1MOUSE, STRING_INIT("X1Mouse"), NULL},
-    {TRUE, K_X1RELEASE, STRING_INIT("X1Release"), NULL},
-    {TRUE, K_X2DRAG, STRING_INIT("X2Drag"), NULL},
-    {TRUE, K_X2MOUSE, STRING_INIT("X2Mouse"), NULL},
-    {TRUE, K_X2RELEASE, STRING_INIT("X2Release"), NULL},
-    {TRUE, K_CSI, STRING_INIT("xCSI"), NULL},
-    {TRUE, K_XDOWN, STRING_INIT("xDown"), NULL},
-    {TRUE, K_XEND, STRING_INIT("xEnd"), NULL},
-    {TRUE, K_XF1, STRING_INIT("xF1"), NULL},
-    {TRUE, K_XF2, STRING_INIT("xF2"), NULL},
-    {TRUE, K_XF3, STRING_INIT("xF3"), NULL},
-    {TRUE, K_XF4, STRING_INIT("xF4"), NULL},
-    {TRUE, K_XHOME, STRING_INIT("xHome"), NULL},
-    {TRUE, K_XLEFT, STRING_INIT("xLeft"), NULL},
-    {TRUE, K_XRIGHT, STRING_INIT("xRight"), NULL},
-    {TRUE, K_XUP, STRING_INIT("xUp"), NULL},
-    {TRUE, K_ZEND, STRING_INIT("zEnd"), NULL},
-    {TRUE, K_ZHOME, STRING_INIT("zHome"), NULL}
+	K_URXVT_MOUSE, STRING_INIT("UrxvtMouse"), FALSE},
+    {TRUE, K_X1DRAG, STRING_INIT("X1Drag"), FALSE},
+    {TRUE, K_X1MOUSE, STRING_INIT("X1Mouse"), FALSE},
+    {TRUE, K_X1RELEASE, STRING_INIT("X1Release"), FALSE},
+    {TRUE, K_X2DRAG, STRING_INIT("X2Drag"), FALSE},
+    {TRUE, K_X2MOUSE, STRING_INIT("X2Mouse"), FALSE},
+    {TRUE, K_X2RELEASE, STRING_INIT("X2Release"), FALSE},
+    {TRUE, K_CSI, STRING_INIT("xCSI"), FALSE},
+    {TRUE, K_XDOWN, STRING_INIT("xDown"), FALSE},
+    {TRUE, K_XEND, STRING_INIT("xEnd"), FALSE},
+    {TRUE, K_XF1, STRING_INIT("xF1"), FALSE},
+    {TRUE, K_XF2, STRING_INIT("xF2"), FALSE},
+    {TRUE, K_XF3, STRING_INIT("xF3"), FALSE},
+    {TRUE, K_XF4, STRING_INIT("xF4"), FALSE},
+    {TRUE, K_XHOME, STRING_INIT("xHome"), FALSE},
+    {TRUE, K_XLEFT, STRING_INIT("xLeft"), FALSE},
+    {TRUE, K_XRIGHT, STRING_INIT("xRight"), FALSE},
+    {TRUE, K_XUP, STRING_INIT("xUp"), FALSE},
+    {TRUE, K_ZEND, STRING_INIT("zEnd"), FALSE},
+    {TRUE, K_ZHOME, STRING_INIT("zHome"), FALSE}
     // NOTE: When adding a long name update MAX_KEY_NAME_LEN.
 };
 #undef STRING_INIT
@@ -1234,7 +1207,7 @@ handle_x_keys(int key)
 get_special_key_name(int c, int modifiers)
 {
     static char_u string[MAX_KEY_NAME_LEN + 1];
-    int	    i, idx;
+    int	    i, idx, len;
     int	    table_idx;
 
     string[0] = '<';
@@ -1307,10 +1280,11 @@ get_special_key_name(int c, int modifiers)
 	// Not a special key, only modifiers, output directly
 	else
 	{
-	    if (has_mbyte && (*mb_char2len)(c) > 1)
-		idx += (*mb_char2bytes)(c, string + idx);
-	    else if (vim_isprintc(c))
+	    len = (*mb_char2len)(c);
+	    if (len == 1 && vim_isprintc(c))
 		string[idx++] = c;
+	    else if (has_mbyte && len > 1)
+		idx += (*mb_char2bytes)(c, string + idx);
 	    else
 	    {
 		char_u	*s = transchar(c);
@@ -1323,10 +1297,7 @@ get_special_key_name(int c, int modifiers)
     {
 	string_T    *s;
 
-	if (key_names_table[table_idx].pref_name != NULL)
-	    s = key_names_table[table_idx].pref_name;
-	else
-	    s = &key_names_table[table_idx].name;
+	s = &key_names_table[table_idx].name;
 
 	if (s->length + idx + 2 <= MAX_KEY_NAME_LEN)
 	{
@@ -1717,7 +1688,7 @@ find_special_key_in_table(int c)
     int	    i;
 
     for (i = 0; i < (int)ARRAY_LENGTH(key_names_table); i++)
-	if (c == key_names_table[i].key)
+	if (c == key_names_table[i].key && !key_names_table[i].is_alt)
 	    return key_names_table[i].enabled ? i : -1;
 
     return -1;
@@ -1799,7 +1770,6 @@ get_special_key_code(char_u *name)
 	target.key = 0;
 	target.name.string = name;
 	target.name.length = 0;
-	target.pref_name = NULL;
 
 	entry = (struct key_name_entry *)bsearch(
 	    &target,
@@ -1825,9 +1795,7 @@ get_key_name(int i)
     if (i < 0 || i >= (int)ARRAY_LENGTH(key_names_table))
 	return NULL;
 
-    return (key_names_table[i].pref_name != NULL) ?
-	key_names_table[i].pref_name->string :
-	key_names_table[i].name.string;
+    return key_names_table[i].name.string;
 }
 
 /*
@@ -1906,6 +1874,9 @@ set_fileformat(
     // This may cause the buffer to become (un)modified.
     check_status(curbuf);
     redraw_tabline = TRUE;
+#if defined(FEAT_TABPANEL)
+    redraw_tabpanel = TRUE;
+#endif
     need_maketitle = TRUE;	    // set window title later
 }
 
@@ -2075,8 +2046,7 @@ same_directory(char_u *f1, char_u *f2)
 
 #if defined(FEAT_SESSION) || defined(FEAT_AUTOCHDIR) \
 	|| defined(MSWIN) || defined(FEAT_GUI_GTK) \
-	|| defined(FEAT_NETBEANS_INTG) \
-	|| defined(PROTO)
+	|| defined(FEAT_NETBEANS_INTG)
 /*
  * Change to a file's directory.
  * Caller must call shorten_fnames()!
@@ -2111,7 +2081,7 @@ vim_chdirfile(char_u *fname, char *trigger_autocmd)
 }
 #endif
 
-#if defined(STAT_IGNORES_SLASH) || defined(PROTO)
+#if defined(STAT_IGNORES_SLASH)
 /*
  * Check if "name" ends in a slash and is not a directory.
  * Used for systems where stat() ignores a trailing slash on a file name.
@@ -2141,7 +2111,7 @@ vim_stat(const char *name, stat_T *stp)
 }
 #endif
 
-#if defined(CURSOR_SHAPE) || defined(PROTO)
+#if defined(CURSOR_SHAPE)
 
 /*
  * Handling of cursor and mouse pointer shapes in various modes.
@@ -2449,7 +2419,7 @@ parse_shape_opt(int what)
 }
 
 # if defined(MCH_CURSOR_SHAPE) || defined(FEAT_GUI) \
-	|| defined(FEAT_MOUSESHAPE) || defined(PROTO)
+	|| defined(FEAT_MOUSESHAPE)
 /*
  * Return the index into shape_table[] for the current mode.
  * When "mouse" is TRUE, consider indexes valid for the mouse pointer.
@@ -2502,7 +2472,7 @@ get_shape_idx(int mouse)
 }
 #endif
 
-# if defined(FEAT_MOUSESHAPE) || defined(PROTO)
+# if defined(FEAT_MOUSESHAPE)
 static int current_mouse_shape = 0;
 
 /*
@@ -2556,7 +2526,7 @@ update_mouseshape(int shape_idx)
 
 #endif // CURSOR_SHAPE
 
-#if defined(FEAT_EVAL) || defined(PROTO)
+#if defined(FEAT_EVAL)
 /*
  * Mainly for tests: get the name of the current mouse shape.
  */
@@ -2565,7 +2535,7 @@ f_getmouseshape(typval_T *argvars UNUSED, typval_T *rettv)
 {
     rettv->v_type = VAR_STRING;
     rettv->vval.v_string = NULL;
-# if defined(FEAT_MOUSESHAPE) || defined(PROTO)
+# if defined(FEAT_MOUSESHAPE)
     if (current_mouse_shape >= 0
 			      && current_mouse_shape < (int)MSHAPE_NAMES_COUNT)
 	rettv->vval.v_string = vim_strnsave(
@@ -2621,7 +2591,7 @@ get_user_name(char_u *buf, int len)
     return OK;
 }
 
-#if defined(EXITFREE) || defined(PROTO)
+#if defined(EXITFREE)
 /*
  * Free the memory allocated by get_user_name()
  */
@@ -2632,7 +2602,7 @@ free_username(void)
 }
 #endif
 
-#ifndef HAVE_QSORT
+#if !defined(HAVE_QSORT) && !defined(PROTO)
 /*
  * Our own qsort(), for systems that don't have it.
  * It's simple and slow.  From the K&R C book.
@@ -2699,7 +2669,7 @@ qsort(
  *  (history removed, not very interesting.  See the "screen" sources.)
  */
 
-#if !defined(HAVE_SETENV) && !defined(HAVE_PUTENV)
+#if !defined(HAVE_SETENV) && !defined(HAVE_PUTENV) && !defined(PROTO)
 
 #define EXTRASIZE 5		// increment to add to env. size
 
@@ -2839,7 +2809,7 @@ vimpty_getenv(const char_u *string)
 
 #endif // !defined(HAVE_SETENV) && !defined(HAVE_PUTENV)
 
-#if defined(FEAT_EVAL) || defined(FEAT_SPELL) || defined(PROTO)
+#if defined(FEAT_EVAL) || defined(FEAT_SPELL)
 /*
  * Return 0 for not writable, 1 for writable file, 2 for a dir which we have
  * rights to write into.
@@ -2874,7 +2844,7 @@ filewritable(char_u *fname)
 }
 #endif
 
-#if defined(FEAT_SPELL) || defined(FEAT_PERSISTENT_UNDO) || defined(PROTO)
+#if defined(FEAT_SPELL) || defined(FEAT_PERSISTENT_UNDO)
 /*
  * Read 2 bytes from "fd" and turn them into an int, MSB first.
  * Returns -1 when encountering EOF.
@@ -3016,8 +2986,7 @@ elapsed(DWORD start_tick)
 
 #if defined(FEAT_JOB_CHANNEL) \
 	|| (defined(UNIX) && (!defined(USE_SYSTEM) \
-	|| (defined(FEAT_GUI) && defined(FEAT_TERMINAL)))) \
-	|| defined(PROTO)
+	|| (defined(FEAT_GUI) && defined(FEAT_TERMINAL))))
 /*
  * Parse "cmd" and put the white-separated parts in "argv".
  * "argv" is an allocated array with "argc" entries and room for 4 more.
@@ -3124,7 +3093,7 @@ build_argv_from_string(char_u *cmd, char ***argv, int *argc)
     return OK;
 }
 
-# if defined(FEAT_JOB_CHANNEL) || defined(PROTO)
+# if defined(FEAT_JOB_CHANNEL)
 /*
  * Build "argv[argc]" from the list "l".
  * "argv[argc]" is set to NULL;
@@ -3225,3 +3194,127 @@ cmp_keyvalue_value_ni(const void *a, const void *b)
 		    kv2->value.length));
 }
 
+/*
+ * Iterative merge sort for doubly linked list.
+ * O(NlogN) worst case, and stable.
+ *  - The list is divided into blocks of increasing size (1, 2, 4, 8, ...).
+ *  - Each pair of blocks is merged in sorted order.
+ *  - Merged blocks are reconnected to build the sorted list.
+ */
+    void *
+mergesort_list(
+    void	*head,
+    void	*(*get_next)(void *),
+    void	(*set_next)(void *, void *),
+    void	*(*get_prev)(void *),
+    void	(*set_prev)(void *, void *),
+    int		(*compare)(const void *, const void *))
+{
+    if (!head || !get_next(head))
+	return head;
+
+    // Count length
+    int	    n = 0;
+    void*   curr = head;
+    while (curr)
+    {
+	n++;
+	curr = get_next(curr);
+    }
+
+    int	size;
+    for (size = 1; size < n; size *= 2)
+    {
+	void*	new_head = NULL;
+	void*	tail = NULL;
+	curr = head;
+
+	while (curr)
+	{
+	    // Split two runs
+	    void    *left = curr;
+	    void    *right = left;
+	    int	    i;
+	    for (i = 0; i < size && right; ++i)
+		right = get_next(right);
+
+	    void    *next = right;
+	    for (i = 0; i < size && next; ++i)
+		next = get_next(next);
+
+	    // Break links
+	    void    *l_end = right ? get_prev(right) : NULL;
+	    if (l_end)
+		set_next(l_end, NULL);
+	    if (right)
+		set_prev(right, NULL);
+
+	    void    *r_end = next ? get_prev(next) : NULL;
+	    if (r_end)
+		set_next(r_end, NULL);
+	    if (next)
+		set_prev(next, NULL);
+
+	    // Merge
+	    void    *merged = NULL;
+	    void    *merged_tail = NULL;
+
+	    while (left || right)
+	    {
+		void	*chosen = NULL;
+		if (!left)
+		{
+		    chosen = right;
+		    right = get_next(right);
+		}
+		else if (!right)
+		{
+		    chosen = left;
+		    left = get_next(left);
+		}
+		else if (compare(left, right) <= 0)
+		{
+		    chosen = left;
+		    left = get_next(left);
+		}
+		else
+		{
+		    chosen = right;
+		    right = get_next(right);
+		}
+
+		if (merged_tail)
+		{
+		    set_next(merged_tail, chosen);
+		    set_prev(chosen, merged_tail);
+		    merged_tail = chosen;
+		}
+		else
+		{
+		    merged = merged_tail = chosen;
+		    set_prev(chosen, NULL);
+		}
+	    }
+
+	    // Connect to full list
+	    if (!new_head)
+		new_head = merged;
+	    else
+	    {
+		set_next(tail, merged);
+		set_prev(merged, tail);
+	    }
+
+	    // Move tail to end
+	    while (get_next(merged_tail))
+		merged_tail = get_next(merged_tail);
+	    tail = merged_tail;
+
+	    curr = next;
+	}
+
+	head = new_head;
+    }
+
+    return head;
+}
